@@ -32,46 +32,33 @@
 #include <functional>
 #include <filesystem>
 
-#include <boost/program_options.hpp>
-#include <boost/algorithm/string.hpp>
-// #include <boost/iostreams/filter/bzip2.hpp>
-#include <boost/iostreams/filter/gzip.hpp>
-#include <boost/iostreams/filtering_stream.hpp>
-#include <boost/iostreams/device/file_descriptor.hpp>
-#include <boost/iostreams/copy.hpp>
+#include <cif++.hpp>
+#include <mcfp/mcfp.hpp>
 
-#include "cif++/Cif++.hpp"
-#include "cif++/Cif2PDB.hpp"
-#include "cif++/Structure.hpp"
-#include "cif++/CifParser.hpp"
-#include "cif++/CifValidator.hpp"
-#include "cif++/CifUtils.hpp"
-
-namespace po = boost::program_options;
-namespace ba = boost::algorithm;
 namespace fs = std::filesystem;
-namespace io = boost::iostreams;
-namespace c = mmcif;
 
 using cif::iequals;
 
 // --------------------------------------------------------------------
 
-void updateEntryID(cif::File& target, const std::string& entryID)
+void updateEntryID(cif::file& target, const std::string& entryID)
 {
-	auto& db = target.firstDatablock();
+	auto& db = target.front();
 	
-	if (db.getName() != entryID)
-		db.setName(entryID);
+	if (db.name() != entryID)
+		db.set_name(entryID);
 
 	for (auto r: db["entry"])
 		r["id"] = entryID;
 }
 
-void transplant(cif::File& target, cif::File& donor)
+void transplant(cif::file& target, cif::file& donor)
 {
-	auto& dbt = target.firstDatablock();
-	auto& dbd = donor.firstDatablock();
+	if (target.empty() or donor.empty())
+		throw std::runtime_error("empty files?");
+
+	auto& dbt = target.front();
+	auto& dbd = donor.front();
 
 	for (auto c: { "struct", "struct_keywords", "audit_author", "citation", "citation_author", "diffrn", "diffrn_radiation", "diffrn_radiation_wavelength" })
 	{
@@ -93,16 +80,16 @@ void transplant(cif::File& target, cif::File& donor)
 	}
 	
 	std::string exptlMethod;
-	auto dExplt = dbd["exptl"].find(cif::Key("entry_id") == dbd.getName());
+	auto dExplt = dbd["exptl"].find(cif::key("entry_id") == dbd.name());
 	if (dExplt.size() != 1)
 		throw std::runtime_error("Invalid number of exptl records in donor file, should be exactly one this version of cif-merge");
 	cif::tie(exptlMethod) = dExplt.front().get("method");
-	auto tExplt = dbt["exptl"].find(cif::Key("entry_id") == dbt.getName());
+	auto tExplt = dbt["exptl"].find(cif::key("entry_id") == dbt.name());
 	if (tExplt.empty())
 	{
 		auto c = dbt.emplace("exptl");
 		std::get<0>(c)->emplace({
-			{ "entry_id", dbt.getName() },
+			{ "entry_id", dbt.name() },
 			{ "method", exptlMethod }
 		});
 	}
@@ -127,16 +114,16 @@ void transplant(cif::File& target, cif::File& donor)
 		
 		if (iequals(type, "polymer"))
 		{
-			auto t = dbt["entity_poly"][cif::Key("entity_id") == id];
+			auto t = dbt["entity_poly"].find1(cif::key("entity_id") == id);
 			
 			std::string polyType, seq;
 			cif::tie(polyType, seq) = t.get("type", "pdbx_seq_one_letter_code");
 			
-			auto d = dbd["entity_poly"][cif::Key("type") == polyType and cif::Key("pdbx_seq_one_letter_code") == seq];
+			auto d = dbd["entity_poly"].find1(cif::key("type") == polyType and cif::key("pdbx_seq_one_letter_code") == seq);
 			
 			if (d.empty())
 			{
-				if (cif::VERBOSE)
+				if (cif::VERBOSE > 0)
 					std::cerr << "Cannot map entity " << id << " in target file to an entity in the donor" << std::endl;
 				continue;
 			}
@@ -145,7 +132,7 @@ void transplant(cif::File& target, cif::File& donor)
 			
 			// copy over refseq
 			
-			auto sr = dbd["struct_ref"][cif::Key("entity_id") == dEntityID];
+			auto sr = dbd["struct_ref"].find1(cif::key("entity_id") == dEntityID);
 			if (not sr.empty())
 			{
 				sr["entity_id"] = id;
@@ -153,22 +140,22 @@ void transplant(cif::File& target, cif::File& donor)
 				
 				std::string refID = sr["id"].as<std::string>();
 				
-				for (auto r: dbd["struct_ref_seq"].find(cif::Key("ref_id") == refID))
+				for (auto r: dbd["struct_ref_seq"].find(cif::key("ref_id") == refID))
 					dbt["struct_ref_seq"].emplace(r);
 			}
 		}
 		else if (iequals(type, "non-polymer"))
 		{
-			auto t = dbt["pdbx_entity_nonpoly"][cif::Key("entity_id") == id];
+			auto t = dbt["pdbx_entity_nonpoly"].find1(cif::key("entity_id") == id);
 			
 			std::string compID;
 			cif::tie(compID) = t.get("comp_id");
 			
-			auto d = dbd["pdbx_entity_nonpoly"][cif::Key("comp_id") == compID];
+			auto d = dbd["pdbx_entity_nonpoly"].find1(cif::key("comp_id") == compID);
 			
 			if (d.empty())
 			{
-				if (cif::VERBOSE)
+				if (cif::VERBOSE > 0)
 					std::cerr << "Cannot map entity " << id << " in target file to an entity in the donor" << std::endl;
 				continue;
 			}
@@ -177,17 +164,16 @@ void transplant(cif::File& target, cif::File& donor)
 		}
 		else if (iequals(type, "water"))
 		{
-			cif::tie(dEntityID) = dbd["entity"][cif::Key("type") == type].get("id");
+			dEntityID = dbd["entity"].find1<std::string>(cif::key("type") == type, "id");
 		}
-		else if (cif::VERBOSE)
+		else if (cif::VERBOSE > 0)
 			std::cerr << "Unsupported entity type: " << type << std::endl;
 		
 		if (dEntityID.empty())
 			continue;
 
-		std::string srcMethod, description, weight;
-		cif::tie(srcMethod, description, weight) =
-			dbd["entity"][cif::Key("id") == dEntityID].get("src_method", "pdbx_description", "formula_weight");
+		const auto &[srcMethod, description, weight] =
+			dbd["entity"].find1<std::string,std::string,std::string>(cif::key("id") == dEntityID, "src_method", "pdbx_description", "formula_weight");
 		
 		r["src_method"] = srcMethod;
 		r["pdbx_description"] = description;
@@ -197,7 +183,7 @@ void transplant(cif::File& target, cif::File& donor)
 		{
 			std::string srcRec = kSrcMap.at(srcMethod);
 
-			auto d = dbd[srcRec][cif::Key("entity_id") == dEntityID];
+			auto d = dbd[srcRec].find1(cif::key("entity_id") == dEntityID);
 			if (not d.empty())
 			{
 				d["entity_id"] = id;
@@ -210,67 +196,58 @@ void transplant(cif::File& target, cif::File& donor)
 
 int pr_main(int argc, char* argv[])
 {
-	po::options_description visible_options("cif-merge [options] inputFile donorFile ");
-	visible_options.add_options()
-		("help,h",									"Display help message")
-		("version",									"Print version")
-		("verbose,v",								"Verbose output")
-		("input,i",		po::value<std::string>(),	"Modified PDB file")
-		("output,o",	po::value<std::string>(),	"Output file, default is stdout (terminal)")
-		("donor",		po::value<std::string>(),	"CIF file (or PDB ID for this file) containing the data to collect data from")
-		("dict",		po::value<std::string>(),	"Dictionary file containing restraints for residues in this specific target")
-		;
-	
+	auto &config = mcfp::config::instance();
 
-	po::options_description hidden_options("hidden options");
-	hidden_options.add_options()
-		("debug,d",				po::value<int>(),		"Debug level (for even more verbose output)");
+	config.init(
+		"cif-merge input-file donor-file [output-file]",
+		mcfp::make_option("help,h", "Display help message"),
+		mcfp::make_option("version", "Print version"),
+		mcfp::make_option("verbose,v", "Verbose output"),
+		mcfp::make_hidden_option<int>("debug,d", "Debug level (for even more verbose output)")
+	);
 
-	po::options_description cmdline_options;
-	cmdline_options.add(visible_options).add(hidden_options);
+	config.parse(argc, argv);
 
-	po::positional_options_description p;
-	p.add("input", 1);
-	p.add("donor", 1);
-	
-	po::variables_map vm;
-	po::store(po::command_line_parser(argc, argv).options(cmdline_options).positional(p).run(), vm);
-	po::notify(vm);
-
-	if (vm.count("version"))
+	if (config.has("version"))
 	{
-		write_version_string(std::cout, vm.count("verbose"));
+		write_version_string(std::cout, config.has("verbose"));
 		exit(0);
 	}
 
-	if (vm.count("help") or vm.count("input") == 0 or vm.count("donor") == 0)
+	if (config.has("help") or config.operands().size() < 2)
 	{
-		std::cerr << visible_options << std::endl;
-		exit(1);
+		std::cerr << config << std::endl;
+		exit(config.has("help") ? 0 : 1);
 	}
 
-	cif::VERBOSE = vm.count("verbose") != 0;
-	if (vm.count("debug"))
-		cif::VERBOSE = vm["debug"].as<int>();
+	cif::VERBOSE = config.count("verbose");
+	if (config.has("debug"))
+		cif::VERBOSE = config.get<int>("debug");
 
 	// Load dict, if any
 	
-	if (vm.count("dict"))
-		c::CompoundFactory::instance().pushDictionary(vm["dict"].as<std::string>());
+	// if (vm.count("dict"))
+	// 	c::CompoundFactory::instance().pushDictionary(vm["dict"].as<std::string>());
 
 	// Read input file
-	mmcif::File cf{vm["input"].as<std::string>()};
+	cif::gzio::ifstream in(config.operands().front());
+	if (not in.is_open())
+		throw std::runtime_error("Could not open input file");
 	
-	// Read donor file
-	mmcif::File df{vm["donor"].as<std::string>()};
+	cif::gzio::ifstream donor(config.operands()[1]);
+	if (not donor.is_open())
+		throw std::runtime_error("Could not open donor file");
+	
+	cif::file cf{in};
+	cif::file df{donor};
 
-	updateEntryID(cf.file(), df.data().getName());
-	transplant(cf.file(), df.file());
+	updateEntryID(cf, df.front().name());
+	transplant(cf, df);
 	
-	if (vm.count("output"))
-		cf.save(vm["output"].as<std::string>());
+	if (config.operands().size() == 3)
+		cf.save(config.operands().back());
 	else
-		cf.file().save(std::cout);
+		cf.save(std::cout);
 
 	return 0;	
 }
